#!/usr/bin/env python
# -*- test-case-name: test_smoker -*-

# Copyright (c) 2011 Marco Giusti
# Copyright (c) Twisted Matrix Laboratories.

import os
import sys
import itertools


def _parseLocalVariables(line):
    """
    Accepts a single line in Emacs local variable declaration format and
    returns a dict of all the variables {name: value}.
    Raises ValueError if 'line' is in the wrong format.

    See http://www.gnu.org/software/emacs/manual/html_node/File-Variables.html
    """
    paren = '-*-'
    start = line.find(paren) + len(paren)
    end = line.rfind(paren)
    if start == -1 or end == -1:
        raise ValueError("%r not a valid local variable declaration" % (line,))
    items = line[start:end].split(';')
    localVars = {}
    for item in items:
        if len(item.strip()) == 0:
            continue
        split = item.split(':')
        if len(split) != 2:
            raise ValueError("%r contains invalid declaration %r"
                             % (line, item))
        localVars[split[0].strip()] = split[1].strip()
    return localVars


def loadLocalVariables(filename):
    """
    Accepts a filename and attempts to load the Emacs variable declarations
    from that file, simulating what Emacs does.

    See http://www.gnu.org/software/emacs/manual/html_node/File-Variables.html
    """
    f = file(filename, "r")
    lines = [f.readline(), f.readline()]
    f.close()
    for line in lines:
        try:
            return _parseLocalVariables(line)
        except ValueError:
            pass
    return {}


def getTestModules(filename):
    testCaseVar = loadLocalVariables(filename).get('test-case-name', None)
    if testCaseVar is None:
        return []
    return testCaseVar.split(',')


def main(*args, **kwds):
    if sys.version_info < (2, 7):
        import unittest2 as unittest
    else:
        import unittest

    return unittest.main(*args, **kwds)


def runSmokeTests(ui, repo, node, **kwds):
    files = (os.path.join(repo.root, fn) for fn in repo[node].files())
    files = filter(os.path.exists, files)
    argv = list(itertools.chain.from_iterable(map(getTestModules, files)))
    if not argv:
        return False
    argv.insert(0, 'test_smoker.py')
    program = main(module=None, argv=argv, exit=False)
    return not program.result.wasSuccessful()


def externalHook():
    from mercurial import hg, ui
    ui = ui.ui()
    repo = hg.repository(ui, '.')
    return runSmokeTests(ui, repo, os.environ['HG_NODE'])


if __name__ == '__main__':
    sys.exit(externalHook())
