#!/usr/bin/env python

import sys
import os
import tempfile
import shutil
if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

import mock

import smoker


class TempDir(object):

    def setUp(self):
        self.tmp = tempfile.mkdtemp()
        self._files = []
        super(TempDir, self).setUp()

    def tearDown(self):
        if os.path.exists(self.tmp):
            shutil.rmtree(self.tmp, True)
        map(file.close, self._files)
        super(TempDir, self).tearDown()

    def makeTempFile(self, mode='wr'):
        fd, fn = tempfile.mkstemp(dir=self.tmp)
        os.close(fd)
        fp = open(fn, mode)
        self._files.append(fp)
        return fp


class TestGatherTestCases(unittest.TestCase):

    @unittest.skipIf(not os.path.exists('smoker.py'), 'cannot find smoker.py')
    def test_gatherSmokeTests(self):
        tests = smoker.getTestModules('smoker.py')
        self.assertSequenceEqual(tests, ['test_smoker'])


class TestInProcess(TempDir, unittest.TestCase):

    def setUp(self):
        super(TestInProcess, self).setUp()
        repo = mock.MagicMock()
        repo.root = self.tmp
        self.f1 = self.makeTempFile()
        self.f2 = self.makeTempFile()
        self.f3 = self.makeTempFile()
        self.ctx = mock.Mock()
        self.files = map(os.path.basename, [self.f1.name, self.f2.name,
                                            self.f3.name])
        self.ctx.files = mock.Mock(return_value=self.files)
        repo.__getitem__.return_value = self.ctx

        self.repo = repo
        self._main = smoker.main
        self.main = smoker.main = mock.Mock()

    def tearDown(self):
        super(TestInProcess, self).tearDown()
        smoker.main = self._main

    @unittest.skip('in progress')
    def test_simpleTest(self):
        smoker.main.return_value = False
        self.assertTrue(smoker.runSmokeTests(None, self.repo, None))

    def test_noTests(self):
        smoker.runSmokeTests(None, self.repo, None)
        self.assertFalse(self.main.called)

    def test_argv(self):
        with self.f1 as fp:
            self.f1.write("# -*- test-case-name: test1 -*-")
        result = smoker.runSmokeTests(None, self.repo, None)
        self.assertTrue(self.main.called)
        args = {'module': None, 'argv': ['test_smoker.py', 'test1'],
                'exit': False}
        self.assertDictEqual(self.main.call_args[1], args)

    def assertNotRaises(self, excClass, callableObj, *args, **kwargs):
        try:
            callableObj(*args, **kwargs)
        except excClass:
            if hasattr(excClass,'__name__'):
                excName = excClass.__name__
            else:
                excName = str(excClass)
            raise self.failureException, "%s raised" % excName
        return

    def test_removedFiles(self):
        filename = self.f1.name
        self.f1.close()
        os.unlink(filename)
        self.assertNotRaises(IOError, smoker.runSmokeTests, None, self.repo,
                             None)


if __name__ == '__main__':
    unittest.main()
